@@ -5,9 +5,81 @@ import os
 import warnings
 import tempfile
 from pathlib import Path
+import ctypes
+import sys
+import tomllib as toml
+import fnmatch
+import pkgutil
+import importlib
+import struct
+
+spec = importlib.util.find_spec("restructuredpython")
+if spec and spec.origin:
+    package_dir = os.path.dirname(spec.origin)
+    io_dll = os.path.join(package_dir, "lib", "windows-libs", "io64.dll")
+    
+    io32_dll = os.path.join(package_dir, "lib", "windows-lib", "io32.dll")
+
+    io_so = os.path.join(package_dir, "lib", "linux-libs", "io.so")
+
+    io_dylib = os.path.join(package_dir, "lib", "macos-libs", "io.dylib")
+
+if sys.platform == "win32":
+    if (struct.calcsize("P") * 8) == 32:
+        lib = ctypes.WinDLL(io32_dll)
+    else:
+        lib = ctypes.WinDLL(io_dll)
+elif sys.platform == "darwin":
+    lib = ctypes.CDLL(io_dylib)
+else:
+    lib = ctypes.CDLL(io_so)
+
+
+lib.check_file_exists.argtypes = [ctypes.c_char_p]
+lib.check_file_exists.restype = ctypes.c_int
+
+lib.read_file.argtypes = [ctypes.c_char_p]
+lib.read_file.restype = ctypes.c_char_p
+
+lib.write_file.argtypes = [ctypes.c_char_p, ctypes.c_char_p]
+lib.write_file.restype = ctypes.c_int
+
+lib.read_binary_file.argtypes = [
+    ctypes.c_char_p, ctypes.POINTER(
+        ctypes.c_size_t)]
+lib.read_binary_file.restype = ctypes.POINTER(ctypes.c_char)
+
+
+def load_toml_binary(filename):
+    filename = str(filename)
+    size = ctypes.c_size_t()
+    raw_data_ptr = lib.read_binary_file(filename.encode(), ctypes.byref(size))
+
+    if not raw_data_ptr:
+        raise FileNotFoundError(f"Could not read {filename}")
+
+    raw_data = ctypes.string_at(raw_data_ptr, size.value)
+    return toml.loads(raw_data.decode())
+def read_file_utf8(filename: str) -> str:
+    size = ctypes.c_size_t()
+    filename_bytes = filename.encode('utf-8')
+    
+    ptr = lib.read_binary_file(filename_bytes, ctypes.byref(size))
+    if not ptr:
+        raise FileNotFoundError(f"File not found: {filename}")
+    
+    raw_bytes = ctypes.string_at(ptr, size.value)
+
+    try:
+        text = raw_bytes.decode('utf-8')
+    except UnicodeDecodeError as e:
+        raise ValueError(f"File is not valid UTF-8: {e}")
+    
+    return text
+
 
 token_specification = [
-    ('COMMENT', r'/\*.*?\*/'),  # Multiline comment pattern
+    ('COMMENT', r'/\*.*?\*/'),
     ('IF', r'if'),
     ('FOR', r'for'),
     ('WHILE', r'while'),
@@ -20,8 +92,8 @@ token_specification = [
     ('WITH', r'with'),
     ('MATCH', r'match'),
     ('CASE', r'case'),
-    ('PIPE', r'\|>'),  # pipeline operator
-    ('IDENT', r'[A-Za-z_][A-Za-z0-9_]*'),  # variable or function name
+    ('PIPE', r'\|>'),
+    ('IDENT', r'[A-Za-z_][A-Za-z0-9_]*'),
     ('NUMBER', r'\d+'),
     ('LBRACE', r'\{'),
     ('RBRACE', r'\}'),
@@ -44,22 +116,18 @@ def tokenize(code):
         value = mo.group()
 
         if kind == 'SKIP':
-            continue  # Skip over whitespace and newlines
+            continue
 
         elif kind == 'COMMENT':
-            # Handle multiline comments, converting them to Python-style
-            # comments
             if value.startswith('/*') and value.endswith('*/'):
-                # This is a single multiline comment
-                comment_lines = value[2:-2].splitlines()  # Remove /* and */
+                comment_lines = value[2:-2].splitlines()
                 for line in comment_lines:
                     yield 'COMMENT', f"# {line.strip()}"
-                continue  # Skip the rest of the processing for this comment
-
+                continue
         elif kind == 'MISMATCH':
             warnings.warn(
-                f'Unexpected character {
-                    value!r}. Continuing with compilation')
+                f"Unexpected character {
+                    value!r}. Continuing with compilation")  # fmt: skip
             yield kind, value
 
         else:
@@ -73,23 +141,23 @@ def check_syntax(input_lines):
         if line.startswith(('} else', '} elif')):
             raise SyntaxError(
                 f"Misplaced '{line}' statement at line {
-                    i + 1}. (REPY-0001)")
+                    i + 1}. (REPY-0001)")  # fmt: skip
         if line.startswith('} except'):
             raise SyntaxError(
                 f"Misplaced 'except' statement at line {
-                    i + 1}. (REPY-0002)")
+                    i + 1}. (REPY-0002)")  # fmt: skip
         if line.startswith('} def'):
             raise SyntaxError(
                 f"Misplaced 'def' statement at line {
-                    i + 1}. (REPY-0003)")
+                    i + 1}. (REPY-0003)")  # fmt: skip
         if line.startswith('} class'):
             raise SyntaxError(
                 f"Misplaced 'class' statement at line {
-                    i + 1}. (REPY-0004)")
+                    i + 1}. (REPY-0004)")  # fmt: skip
         if line.startswith('} case'):
             raise SyntaxError(
                 f"Misplaced 'case' statement at line {
-                    i + 1}. (REPY-0005)")
+                    i + 1}. (REPY-0005)")  # fmt: skip
 
 
 def parse_repython(code):
@@ -103,14 +171,11 @@ def parse_repython(code):
                 else:
                     return f'{parts[-1]}({nest(parts[:-1])})'
 
-            # Separate the variable assignment from the pipeline
             variable, pipeline = parts[0].split(
                 '=') if '=' in parts[0] else ('', parts[0])
             variable = variable.strip()
             pipeline = pipeline.strip()
 
-            # If there's a variable assignment, construct the nested calls,
-            # otherwise just return the pipeline
             if variable:
                 nested_call = nest([pipeline] + parts[1:])
                 return f'{variable} = {nested_call}'
@@ -130,38 +195,26 @@ def parse_repython(code):
     for line in lines:
         processed_line = chain_pipeline(line)
 
-        # Handling multiline comments properly
         if inside_comment_block:
             if processed_line.endswith("*/"):
-                # End of multiline comment block
-                # Remove the ending '*/'
                 modified_code.append(f"# {processed_line[:-2].strip()}")
                 inside_comment_block = False
             else:
-                # Continuation of the multiline comment
                 modified_code.append(f"# {processed_line.strip()}")
         elif processed_line.startswith("/*") and processed_line.endswith("*/"):
-            # Single-line comment
-            # Remove the '/*' and '*/'
             modified_code.append(f"# {processed_line[2:-2].strip()}")
         elif processed_line.startswith("/*"):
-            # Beginning of a multiline comment
-            # Remove the starting '/*'
             modified_code.append(f"# {processed_line[2:].strip()}")
             inside_comment_block = True
         elif processed_line.endswith("*/"):
-            # Ending of a multiline comment
-            # Remove the ending '*/'
             modified_code.append(f"# {processed_line[:-2].strip()}")
         else:
-            # Regular code lines (not in a comment)
             if re.match(
                 r'^\s*(if|for|while|def|try|elif|else|except|class|match|with|case)\s.*\{',
                     processed_line):
                 modified_code.append(processed_line.split('{')[0] + ':')
                 brace_stack.append('{')
                 inside_block = True
-            # Handle the case where we encounter a closing brace
             elif re.match(r'^\s*\}', processed_line) and inside_block:
                 brace_stack.pop()
                 inside_block = len(brace_stack) > 0
@@ -182,11 +235,11 @@ def parse_repython(code):
 def compile_header_file(header_filename):
     """Compiles a .cdata file and returns the corresponding Python code."""
     header_filename = Path(header_filename).resolve()
-    if not header_filename.exists():
+    header_filename = str(header_filename)
+    if lib.check_file_exists(header_filename.encode()) == 0:
         raise FileNotFoundError(f"Header file {header_filename} not found.")
     try:
-        with header_filename.open('r') as f:
-            header_code = f.read()
+        header_code = read_file_utf8(header_filename)
         if not header_code.strip():
             raise ValueError(f"Header file {header_filename} is empty.")
     except Exception as e:
@@ -207,22 +260,21 @@ def process_includes(code, input_file):
     header_code = ""
 
     for include in includes:
-        # Handle predefined headers
+        # predefined
         predefined_path = os.path.join(
             PREDEFINED_HEADERS_DIR, include.replace(
                 '.', os.sep) + '.py')
-        if os.path.exists(predefined_path):
+        if lib.check_file_exists(predefined_path.encode()):
             header_code += compile_header_file(predefined_path) + "\n"
             continue
-
-        # Handle user-defined headers
+        # userdefined
         if not os.path.isabs(include):
             include = os.path.join(
                 os.path.dirname(
                     os.path.abspath(input_file)),
                 include)
 
-        if os.path.exists(include):
+        if lib.check_file_exists(include.encode()):
             header_code += compile_header_file(include) + "\n"
         else:
             print(f"Error: Included file '{include}' not found.")
@@ -236,8 +288,7 @@ def process_includes(code, input_file):
 def execute_code_temporarily(code):
     with tempfile.TemporaryDirectory() as tmpdir:
         temp_file_path = os.path.join(tmpdir, "compiled_repy.py")
-        with open(temp_file_path, 'w') as temp_file:
-            temp_file.write(code)
+        lib.write_file(temp_file_path.encode(), code.encode())
         try:
             exec(open(temp_file_path).read(), {"__name__": "__main__"})
         except Exception as e:
@@ -252,22 +303,21 @@ def main():
 
     input_file = args.filename
 
-    if not os.path.exists(input_file):
+    input_file = str(input_file)
+
+    if lib.check_file_exists(input_file.encode()) == 0:
         print(f"Error: The file {input_file} does not exist.")
         return
     if input_file.endswith('repyconfig.toml'):
-        import tomllib as toml
-        import fnmatch
-        with open(input_file, "rb") as file:
-            data = toml.load(file)
+        data = load_toml_binary(input_file)
         try:
             compile_value = data["config"]["compile"]
-        except:
+        except BaseException:
             compile_value = "null"
             print("[WARNING] Error reading compile value from config")
         try:
             exclude_files = data["config"]["exclude"]
-        except:
+        except BaseException:
             exclude_files = []
             print("[WARNING] No excluded files found in config")
         if compile_value == 'all':
@@ -277,14 +327,16 @@ def main():
                 for filename in filenames:
                     if filename.lower().endswith(extension.lower()):
                         file_path_temp = os.path.join(dirpath, filename)
-                        if any(fnmatch.fnmatch(file_path_temp, pattern) for pattern in exclude_files):
+                        if any(fnmatch.fnmatch(file_path_temp, pattern)
+                               for pattern in exclude_files):
                             continue
                         matching_files.append(file_path_temp)
             for file_path_z in matching_files:
-                with open(file_path_z, 'r') as z:
-                    source_code_z = z.read()
+                file_path_z = str(file_path_z)
+                source_code_z = lib.read_file(file_path_z.encode()).decode()
 
-                header_code_z, code_without_includes_z = process_includes(source_code_z, file_path_z)
+                header_code_z, code_without_includes_z = process_includes(
+                    source_code_z, file_path_z)
 
                 python_code_z = parse_repython(code_without_includes_z)
 
@@ -292,12 +344,12 @@ def main():
 
                 output_file_z = os.path.splitext(file_path_z)[0] + '.py'
 
-                with open(output_file_z, 'w') as z:
-                    z.write(final_code_z)
-                print(f"[DEBUG] Successfully compiled {file_path_z} to {output_file_z}")
+                lib.write_file(output_file_z.encode(), final_code_z.encode())
+
+                print(
+                    f"[DEBUG] Successfully compiled {file_path_z} to {output_file_z}")
     else:
-        with open(input_file, 'r') as f:
-            source_code = f.read()
+        source_code = lib.read_file(input_file.encode()).decode()
 
         header_code, code_without_includes = process_includes(
             source_code, input_file)
@@ -308,8 +360,7 @@ def main():
 
         output_file = os.path.splitext(input_file)[0] + '.py'
 
-        with open(output_file, 'w') as f:
-            f.write(final_code)
+        lib.write_file(output_file.encode(), final_code.encode())
 
         print(f"[DEBUG] Successfully compiled {input_file} to {output_file}")
 
@@ -320,13 +371,13 @@ def launch():
     args = parser.parse_args()
 
     input_file = args.filename
+    input_file = str(input_file)
 
-    if not os.path.exists(input_file):
+    if lib.check_file_exists(input_file.encode()) == 0:
         print(f"Error: The file {input_file} does not exist.")
         return
 
-    with open(input_file, 'r') as f:
-        source_code = f.read()
+    source_code = lib.read_file(input_file.encode()).decode()
 
     header_code, code_without_includes = process_includes(
         source_code, input_file)
@@ -335,7 +386,6 @@ def launch():
 
     final_code = header_code + python_code
 
-    # Execute the compiled code directly
     try:
         execute_code_temporarily(final_code)
     except Exception as e:
